/*
    tstore - an archive format for archiving tar archives
    Copyright (C) 2004,2005  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _TSTORE_H
#define _TSTORE_H

#define _GNU_SOURCE
#define _LARGEFILE_SOURCE  
#define _LARGEFILE64_SOURCE  
#define _FILE_OFFSET_BITS 64

#define _XOPEN_SOURCE 500

#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <assert.h>

#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/time.h>

#include "tsdefs.h"

uint64_t nextprime(uint64_t p);
uint64_t prevprime(uint64_t p);

extern uint64_t buckets;
extern uint64_t pri_buckets;
extern uint64_t sec_buckets;
extern uint8_t hash_key[16];
void init_hash(void);
uint64_t hash_sector(uint8_t *data);
int put_hash(uint8_t *bucket, uint64_t bnum, uint16_t small_hash, int *max);
void hash_advice(int overflows, int maxused, int maxsecused);
void hash_buf(uint8_t *buf, int size);
ssize_t read_and_hash(int fd, uint8_t *buf);

uint64_t size_in_blocks(int fd);
uint64_t test_store_size(int fd, int p);
void store_error(uint64_t size, char *name);
ssize_t myread(int fd, void *buf, size_t count);
int filelevels(uint64_t size);
int seqblock(uint8_t *buf);
void get_block(uint64_t n, uint8_t *buf, char *store_name, int store_fd);

uint64_t get_nbyteint(uint8_t *p, int s);
void put_nbyteint(uint8_t *p, int s, uint64_t n);

/* Hashed block size a 512 byte blocked prefixed with a 16 byte hash */
#define HBS (16+512)

typedef struct wtree *wtree_t;
wtree_t insert(wtree_t *tp, wtree_t newnode);
wtree_t find(wtree_t t, uint8_t *elm);
void discard(wtree_t t);
uint64_t *locationp(wtree_t t);
uint8_t *datap(wtree_t t);
wtree_t allocnode();
wtree_t find_hash(wtree_t t, uint32_t h);

void errstop(const char *s);

static inline void *myalloc(size_t size)
{
  void *r=calloc(1,size);
  if (!r) errstop("malloc");
  return r;
}
#define NEW(v) v=myalloc(sizeof(*(v)))

#endif
